#-----------------------------------------------------------
#create master clock
#-----------------------------------------------------------

#FPGA clock pin name connected to S/LABS HBMC refrence clk (in_clk)
derive_pll_clocks -create_base_clocks

#-----------------------------------------------------------
#Set clock path
#-----------------------------------------------------------

   if {"device_family" == "Arria 10"      || "device_family" == "ARRIA 10" || 
       "device_family" == "Cyclone 10 GX" || "device_family" == "CYCLONE 10 GX"} {
      set hyperbus_clk    "*ltpll_0|*clk_0"
      set core_clk        "*ltpll_0|*clk_core"
    } elseif {"device_family" == "Cyclone V"     || "device_family" == "CYCLONE V" } {
      set hyperbus_clk    "*ltpll_0|*general[0].*|divclk"
      set core_clk        "*ltpll_0|*general[2].*|divclk"
 
      if { [get_collection_size [get_clocks ${hyperbus_clk}]] < 1} {
        set hyperbus_clk    "*ltpll_0|*counter[0].*|divclk"
      }

      if { [get_collection_size [get_clocks ${core_clk}]] < 1} {
        set core_clk        "*ltpll_0|*counter[2].*|divclk"
      }
    } else {   
      set hyperbus_clk    "*ltpll_0|*|*|clk[0]"
      set core_clk        "*ltpll_0|*|*|clk[2]"
    }   

   if { [get_collection_size [get_clocks ${hyperbus_clk}]] < 1} {
    return -code error "No clock match $hyperbus_clk found"           
   }
    
   if {single_clock_mode == "false"} {
      if { [get_collection_size [get_clocks ${core_clk}]] < 1} {
        return -code error "No clock match $core_clk found"           
      }
   }   



#-----------------------------------------------------------
#output clock signal name
#-----------------------------------------------------------

   if {"device_family" == "ARRIA 10"  || "device_family" == "Arria 10" || "device_family" == "Cyclone 10 GX" || "device_family" == "CYCLONE 10 GX"} {
    set clkout_pin_name  "*|U_IO|U_CLK0|*|*|output_buffer.obuf\|o"
    set clknout_pin_name "*|U_IO|U_CLK0n|*|*|output_buffer.obuf\|o"
   } elseif {"device_family" == "Max 10"     || "device_family" == "MAX 10" } { 
    set clkout_pin_name  "*|U_IO|U_CLK0|*|*|output_buf.obuf\|o"
    set clknout_pin_name "*|U_IO|U_CLK0n|*|*|output_buf.obuf\|o"
   } else { 
    set clkout_pin_name  "*|U_IO|U_CLK0|*|*|ddio_outa\[0\]\|dataout"    
    set clknout_pin_name "*|U_IO|U_CLK0n|*|*|ddio_outa\[0\]\|dataout"    
   }

#-----------------------------------------------------------
# Output delay
#-----------------------------------------------------------
set HB_clk_ports     [get_ports -no_case *HB*clk*]
set HB_csn_ports     [get_ports -no_case *HB*cs* ]

set_max_delay -from [get_registers *] -to ${HB_csn_ports}   [expr (0.6* dqs_clk_freq_in_ns)]
set_max_delay -from [get_registers *] -to ${HB_clk_ports}   [expr (0.6* dqs_clk_freq_in_ns)]

#-----------------------------------------------------------
# Board parameters - RWDS input maximum/minimum delay
#-----------------------------------------------------------
#
set dqs_dq_skew_list "dqs_dq_skew_info"



# AN 433: Page 24
#   Altera recommends constraining the maximum data invalid time instead of the
#   minimum data valid time. 
#
#   To constrain the maximum data invalid time, you must set
#   up the output minimum delay and output maximum delay constraints. The output
#   minimum delay constraint value is the positive skew requirement, and the output
#   maximum delay constraint is the negative skew requirement. Depending on the
#   operation of your interface, you may have to adjust the constraint values or add
#   exceptions to ensure correct timing analysis, but there are no calculations required to
#   determine the initial output delay values.
#   The output minimum delay shown in Figure 34 corresponds to the minimum tCO
#   requirement, which is the earliest time that data can change after the clock edge.
#   When you constrain the maximum data invalid time, the earliest time that data can
#   change after the clock edge is the positive skew time. The output maximum delay
#   shown in Figure 34 corresponds to the tCO requirement, which is the latest time that
#   data changes after the clock edge. When you constrain the maximum data invalid
#   time, the latest time that data changes occur is skew requirement, which is before the
#   clock edge. To express that time as occurring after the clock edge, you must use the
#   negative slack value.
#   Note:   If you use this approach to constrain the maximum data invalid time and you use the
#           PrimeTime software, you must modify your output maximum and minimum delay
#           values to work around a limitation of the PrimeTime software. Using an output
#           maximum delay value that is less than the corresponding output minimum delay
#           value results in incorrect timing analysis. The following sections describe constraint
#           modifications necessary for each combination of edge capture and alignment. For an
#           appropriate workaround, refer to the section that corresponds to the operation of your
#           interface.
# ...
#   Same-Edge Capture Center-Aligned Output
#   Figure 45 shows the default setup and hold relationships that are analyzed in a
#   same-edge capture, center-aligned output. Red arrows indicate setup relationships,
#   and blue arrows indicate hold relationships.   
#   ...
#   The value you use for the maximum and minimum delay exceptions must include the
#   skew between the clocks. In those cases in which skew is negligible, __such as when you
#   use ALTDDIO_OUT megafunctions for both the data and clock outputs, you can use
#   zero for the skew__. You must not use zero for the skew when a PLL output drives
#   directly off chip, for example.
#
set dq_out_max_dly    0.0  ;#CLK-DOUT input maximum delay
set dq_out_min_dly   -0.0  ;#CLK-DOUT input minimim delay


#-----------------------------------------------------------
#virtual clock with same phase as real clock due to edge aligned input
#-----------------------------------------------------------
create_clock -name virt_rwds_clk -period dqs_clk_freq_in_ns

#derive clocks and clock uncertainty
derive_clock_uncertainty

#get rwds in port names
set rwds_in_ports [get_ports -no_case -nowarn rwds_clk_pin_name]
set rwds_num_pins [ get_collection_size $rwds_in_ports]
set rwds_in_port_list ""
foreach_in_collection name_id  $rwds_in_ports {
    set port_name [get_port_info -name $name_id]
    lappend rwds_in_port_list $port_name
}

#get rwds out port names
set rwds_out_ports [get_ports -no_case -nowarn *HB*rwds]
set rwds_out_port_list ""
foreach_in_collection name_id  $rwds_out_ports {
    set port_name [get_port_info -name $name_id]
    lappend rwds_out_port_list $port_name
}


#-----------------------------------------------------
#get Hbus clk0p or HB_CLK0 out port names
set clk_out_ports [get_ports -no_case -nowarn *HB*_clk0]

if { [get_collection_size $clk_out_ports] < 1} {
set clk_out_ports [get_ports -no_case -nowarn *HB*_clk0p]
 }
   
  

set clk_out_port_list ""
foreach_in_collection name_id  $clk_out_ports {
    set port_name [get_port_info -name $name_id]
    lappend clk_out_port_list $port_name
}


#get clk_out_source pin names
set clk_out_source [get_pins -no_case -compatibility_mode $clkout_pin_name]
set clk_out_source_list ""
foreach_in_collection name_id  $clk_out_source {
    set pin_name [get_pin_info -name $name_id]
    lappend clk_out_source_list $pin_name
}

#-----------------------------------------------------
#get Hbus clk0n  out port names
set clkn_out_ports [get_ports -no_case -nowarn *HB*_clk0n]

if { [get_collection_size $clkn_out_ports] < 1} {
set enable_clkn 0
 }  else {
set enable_clkn 1
}

if {$enable_clkn == 1} {

  set clkn_out_port_list ""
  foreach_in_collection name_id  $clkn_out_ports {
      set port_name [get_port_info -name $name_id]
      lappend clkn_out_port_list $port_name
  }

  #get clk_out_source pin names
  set clkn_out_source [get_pins -no_case -compatibility_mode $clknout_pin_name]
  set clkn_out_source_list ""
  foreach_in_collection name_id  $clkn_out_source {
      set pin_name [get_pin_info -name $name_id]
      lappend clkn_out_source_list $pin_name
  }

}

#-----------------------------------------------------
#get dq port names
set dq_port_list ""
foreach_in_collection name_id [get_ports -no_case -nowarn *HB*dq[*]] {
    set port_name [get_port_info -name $name_id]
    lappend dq_port_list $port_name
}

#
#loop over number of instantiated controllers
for {set hb_cs 0} {$hb_cs < $rwds_num_pins} {incr hb_cs} {

   #
   #set rwds clock pin as either RWDS or RWDSC
   #
   set HB_rwds_in_ports   [lindex $rwds_in_port_list  $hb_cs]
   set HB_rwds_out_ports  [lindex $rwds_out_port_list $hb_cs]
   set HB_clk_out_ports   [lindex $clk_out_port_list $hb_cs]
   set HB_clk_out_source  [lindex $clk_out_source_list $hb_cs]

   #
   #disable timing attribute if pin is not actually connected
   #
   if {$HB_clk_out_source == ""} {
     set disable_clk_out_timing 1
   } else {
     set disable_clk_out_timing 0
   }

   set HB_dq_dqs_skew     [lindex $dqs_dq_skew_list $hb_cs]
   
   set rwds_clk_name      rds_clk${hb_cs}
   set clkout_clk_name    clkout${hb_cs}
   
   #-----------------------------------------------------------
   #virtual input clock with same phase as real clock due to edge aligned input
   #-----------------------------------------------------------
   create_clock -name $rwds_clk_name  -period dqs_clk_freq_in_ns ${HB_rwds_in_ports}
 
   #-----------------------------------------------------------
   #Output Clock -  HB_CLK0-HB_CLK0n
   #-----------------------------------------------------------
   if  {$disable_clk_out_timing==0} {

      create_generated_clock -name $clkout_clk_name   -source ${HB_clk_out_source}   -divide_by 1 -multiply_by 1 -invert ${HB_clk_out_ports}  
      
     	# Clock Uncertainty is accounted for by the ...pathjitter parameters
     	set_clock_uncertainty -to [ get_clocks $clkout_clk_name  ] 0
      
      if {$enable_clkn == 1} {
         set HB_clkn_out_source [lindex $clkn_out_source_list $hb_cs]
         set HB_clkn_out_ports  [lindex $clkn_out_port_list $hb_cs]
         set clknout_clk_name   clknout${hb_cs}
      
         create_generated_clock -name $clknout_clk_name  -source ${HB_clkn_out_source}  -divide_by 1 -multiply_by 1 -invert ${HB_clkn_out_ports}  
   
   	  	# Clock Uncertainty is accounted for by the ...pathjitter parameters
     		set_clock_uncertainty -to [ get_clocks $clknout_clk_name  ] 0
      }

      #-----------------------------------------------------------
      #RWDS timing
      #-----------------------------------------------------------
      set dq_start_idx [expr 8*$hb_cs]
      set dq_end_idx   [expr $dq_start_idx+8]
      
      for {set dq_idx $dq_start_idx} {$dq_idx < $dq_end_idx} {incr dq_idx} {
         set HB_dq_port  [lindex $dq_port_list $dq_idx]
      
         #-----------------------------------------------------------
         #Input Delay
         #-----------------------------------------------------------
         set_input_delay -clock [get_clocks virt_rwds_clk]             -max ${HB_dq_dqs_skew} ${HB_dq_port}
         set_input_delay -clock [get_clocks virt_rwds_clk] -clock_fall -max ${HB_dq_dqs_skew} ${HB_dq_port} -add_delay
      
         set_input_delay -clock [get_clocks virt_rwds_clk]             -min -${HB_dq_dqs_skew} ${HB_dq_port} -add_delay
         set_input_delay -clock [get_clocks virt_rwds_clk] -clock_fall -min -${HB_dq_dqs_skew} ${HB_dq_port} -add_delay
         
         #-----------------------------------------------------------
         #Output Delay
         #-----------------------------------------------------------
         set_output_delay -clock [get_clocks $clkout_clk_name] -min $dq_out_min_dly ${HB_dq_port}
         set_output_delay -clock [get_clocks $clkout_clk_name] -max $dq_out_max_dly ${HB_dq_port}
         set_output_delay -clock [get_clocks $clkout_clk_name] -min $dq_out_min_dly ${HB_dq_port} -clock_fall -add_delay
         set_output_delay -clock [get_clocks $clkout_clk_name] -max $dq_out_max_dly ${HB_dq_port} -clock_fall -add_delay
      
         if {"device_family" == "ARRIA 10"  || "device_family" == "Arria 10" || "device_family" == "Cyclone 10 GX" || "device_family" == "CYCLONE 10 GX"} {      
          #do nothing
         } else {
           set_false_path -from {*sll_hyperbus_controller*U_DQ|*|dffe1a[*]} -to ${HB_dq_port}  
         }
         
#         set_multicycle_path -from {*U_HBC|*|hbmc_io_dq_tri_en} -to ${HB_dq_port} -hold  -end 2
#         set_multicycle_path -from {*U_HBC|*|hbmc_io_dq_tri_en} -to ${HB_dq_port} -setup -end 2
      
      }
   }   

   set_multicycle_path -setup -end -rise_from [get_clocks virt_rwds_clk] -rise_to [get_clocks $rwds_clk_name] 0
   set_multicycle_path -setup -end -fall_from [get_clocks virt_rwds_clk] -fall_to [get_clocks $rwds_clk_name] 0

   set_false_path  -fall_from [get_clocks virt_rwds_clk] -rise_to [get_clocks $rwds_clk_name] -setup
   set_false_path  -rise_from [get_clocks virt_rwds_clk] -fall_to [get_clocks $rwds_clk_name] -setup
   set_false_path  -fall_from [get_clocks virt_rwds_clk] -fall_to [get_clocks $rwds_clk_name] -hold
   set_false_path  -rise_from [get_clocks virt_rwds_clk] -rise_to [get_clocks $rwds_clk_name] -hold

#   set_max_delay -from ${HB_rwds_in_ports} -to [get_registers {*in_refresh180}] 6

 
   #-----------------------------------------------------------
   #setting false paths from RWDS to other clocks
   #-----------------------------------------------------------

   set_false_path -from [get_clocks $rwds_clk_name] -to   [get_clocks ${hyperbus_clk}]
   set_false_path -to   [get_clocks $rwds_clk_name] -from [get_clocks ${hyperbus_clk}]

   if {single_clock_mode == "false"} {
     set_false_path -from [get_clocks $rwds_clk_name] -to   [get_clocks ${core_clk}]
     set_false_path -to   [get_clocks $rwds_clk_name] -from [get_clocks ${core_clk}]
   }
   #-----------------------------------------------------------
   #setting false paths from Core clock to hyperbus clock
   #-----------------------------------------------------------
   if {single_clock_mode == "false"} {
     set_false_path  -from  [get_clocks ${hyperbus_clk}]  -to  [get_clocks ${core_clk}]
     set_false_path  -from  [get_clocks ${core_clk}]      -to  [get_clocks ${hyperbus_clk}]
  }

#   set_multicycle_path -from ${HB_rwds_in_ports} -to [get_registers {*|*|*|rwds_in_0}] -setup -end 2

}

#-----------------------------------------------------------
#On Arria 10, C10GX the oe is registered
#-----------------------------------------------------------

 if {"device_family" == "ARRIA 10"  || "device_family" == "Arria 10" || "device_family" == "Cyclone 10 GX" || "device_family" == "CYCLONE 10 GX"} {     
    set_multicycle_path -from {*U_HBC|*|hbmc_io_dq_tri_en} -to {*U_IO|*U_DQ|*.altera_gpio_bit_i|oe_path.oe_path_fr.oe_reg}   -setup -end 2
    set_multicycle_path -from {*U_HBC|*|hbmc_io_dq_tri_en} -to {*U_IO|*U_DQ|*.altera_gpio_bit_i|oe_path.oe_path_fr.oe_reg}   -hold  -end 2   
 }  

#-----------------------------------------------------------
# On Cyclone V, set false path on the hbmc_io_rds_valid_r signal
#-----------------------------------------------------------
if {"device_family" == "Cyclone V"     || "device_family" == "CYCLONE V" } {
   set_false_path -from [get_registers *hbmc_io_rds_valid_r]
} 

#-----------------------------------------------------------
#setting false paths from inclk to reset 
#-----------------------------------------------------------
set_false_path -from {*iavs0_rstn_*}
set_false_path -from {*hbmc_reset_ff3*}

set_false_path -to   {*iavs0_rstn_*}

