/*
*  DISCLAIMER:
*
*      THIS SOFTWARE, SOURCE CODE AND ASSOCIATED MATERIALS INCLUDING BUT NOT LIMITED TO TUTORIALS,
*      GUIDES AND COMMENTARY PROVIDED WITH THIS EXERCISE ARE ONLY DESIGNED FOR REFERENCE PURPOSES
*      TO GIVE AN EXAMPLE TO LICENSEE FOR THEIR OWN NECESSARY DEVELOPMENT OF THEIR OWN SOFTWARE AND/OR
*      APPLICATION. IT IS NOT DESIGNED FOR ANY SPECIAL PURPOSE, SERIAL PRODUCTION OR USE IN MEDICAL,
*      MILITARY, AIR CRAFT, AVIATION, SPACE OF LIFE SUPPORT EQUIPMENT.
*
*      TO THE EXTENT PERMITTED BY LAW, THE EXERCISE SOFTWARE AND/OR SOURCE CODE AND/OR AND ASSOCIATED
*      MATERIALS IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND AND ONLY FOR REFERENCE PURPOSES.
*
*      SYNAPTIC LABORATORIES LTD. MAKES NO WARRANTIES, EITHER EXPRESS OR IMPLIED, WITH RESPECT TO THE
*      LICENSED SOFTWARE AND/OR SOURCE CODE AND/OR ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND
*      DOCUMENTATION PROVIDED HEREUNDER.
*
*      SYNAPTIC LABORATORIES LTD. SPECIFICALLY DISCLAIMS THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
*      FITNESS FOR A PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT OF ANY INTELLECTUAL
*      PROPERTY RIGHT OF ANY THIRD PARTY WITH REGARD TO THE SOFTWARE, DOCUMENTATION (SCHEMATICS ETC.),
*      SOURCE CODE AND ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND DOCUMENTATION.
*
*      ANY USE, COMPILATION AND TESTING OF THE SOFTWARE AND/OR SOURCE CODE IS AT LICENSEE`S OWN RISK
*      AND LICENSEE IS OBLIGED TO CONDUCT EXTENSIVE TESTS TO AVOID ANY ERRORS AND FAILURE IN THE
*      COMPILED SOURCE CODE, DOCUMENTATION (SCHEMATICS ETC.) AND THE HEREFROM GENERATED SOFTWARE
*      OF LICENSEE.
*
*      EXCEPT FOR WILFULL INTENT SYNAPTIC LABORATORIES LTD. SHALL IN NO EVENT BE ENTITLED TO OR LIABLE
*      FOR ANY INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND OR NATURE, INCLUDING,
*      WITHOUT LIMITATION, BUSINESS INTERRUPTION COSTS, LOSS OF PROFIT OR REVENUE, LOSS OF DATA,
*      PROMOTIONAL OR MANUFACTURING EXPENSES, OVERHEAD, COSTS OR EXPENSES ASSOCIATED WITH WARRANTY
*      OR INTELLECTUAL PROPERTY INFRINGEMENT CLAIMS, INJURY TO REPUTATION OR LOSS OF CUSTOMERS.
*
*/

#ifndef __SLL_AVALON_MBMC_REGS_H__
#define __SLL_AVALON_MBMC_REGS_H__

#include "riscv_io.h"

#define SLL_MBMC_HAL_VERSION "3.2.3"


//------------------------------------------------
// Register addresses (on 32-bit offset) 
//------------------------------------------------
#define IOWR_SLL_MBMC_REG_CMD(base, data)         Risc_Out32(base + 0x00, data)
#define IOWR_SLL_MBMC_REG_IRQ_MASK(base, data)    Risc_Out32(base + 0x0C, data) 
#define IOWR_SLL_MBMC_REG_IRQ_CLR(base, data)     Risc_Out32(base + 0x10, data) 
#define IOWR_SLL_MBMC_REG_CAD_HI(base, data)      Risc_Out32(base + 0x14, data)
#define IOWR_SLL_MBMC_REG_CAD_LO(base, data)      Risc_Out32(base + 0x18, data)
#define IOWR_SLL_MBMC_REG_CAD_DATA(base, data)    Risc_Out32(base + 0x1C, data)


#define IORD_SLL_MBMC_REG_STATUS(base)            Risc_In32(base + 0x00) 
#define IORD_SLL_MBMC_REG_CFG_DEV0(base)          Risc_In32(base + 0x04) 
#define IORD_SLL_MBMC_REG_CFG_DEV1(base)          Risc_In32(base + 0x08) 
#define IORD_SLL_MBMC_REG_IRQ_MASK(base)          Risc_In32(base + 0x0C) 
#define IORD_SLL_MBMC_REG_IRQ_STATUS(base)        Risc_In32(base + 0x10) 
#define IORD_SLL_MBMC_REG_RDATA(base)             Risc_In32(base + 0x1C) 
 
//------------------------------------------------
// Commad Register  bit field 
//
// bit 0 - chip select
// bit 1 - register space mode (0 - memory space : 1 register space)
// bit 2 - r/w mode (0 write : 1 read)
//------------------------------------------------
#define SLL_MBMC_REG_CMD_WR_MODE       0x00	
#define SLL_MBMC_REG_CMD_RD_MODE       0x01

#define SLL_MBMC_REG_CMD_MEM_SPACE     0x00
#define SLL_MBMC_REG_CMD_REG_SPACE     0x02 

#define SLL_MBMC_REG_CMD_CS0           0x00	
#define SLL_MBMC_REG_CMD_CS1           0x04	

#define SLL_MBMC_FLASH_CS0_WR          0x0
#define SLL_MBMC_FLASH_CS0_RD          0x1

#define SLL_MBMC_RAM_CS1_WR            0x6	
#define SLL_MBMC_RAM_CS1_RD            0x7	


//------------------------------------------------
// Status Register  bit fields 
//------------------------------------------------
#define SLL_MBMC_REG_STATUS_HBC_ERR    0x00001
#define SLL_MBMC_REG_STATUS_DEC_ERR    0x00002
#define SLL_MBMC_REG_STATUS_EXT_IRQ    0x00004
#define SLL_MBMC_REG_STATUS_FIFO_ERR   0x00008

#define SLL_MBMC_REG_STATUS_POWER_UP   0x10000  // device still in power up phase
#define SLL_MBMC_REG_STATUS_RDATA_VLD  0x20000  // data returned from memory devce is valid
#define SLL_MBMC_REG_STATUS_CAD_ACTIVE 0x40000  // command accepted and is being processed
                          
//------------------------------------------------
// IRQ MASK Register  bit fields
//------------------------------------------------
#define SLL_MBMC_REG_IRQ_HBC_ERR       0x01
#define SLL_MBMC_REG_IRQ_DEC_ERR       0x02
#define SLL_MBMC_REG_IRQ_EXT_IRQ       0x04
#define SLL_MBMC_REG_IRQ_FIFO_ERR      0x08

//------------------------------------------------
//functions
//------------------------------------------------
extern void     sll_hyperram_reg_write (uint32_t base, uint32_t cad_high, uint32_t cad_low, uint16_t wdata);
extern uint16_t sll_hyperram_reg_read  (uint32_t base, uint32_t cad_high, uint32_t cad_low);

extern void     sll_hyperflash_write_cmd (uint32_t base, uint32_t offset, uint16_t wdata);
extern uint16_t sll_hyperflash_read_cmd  (uint32_t base, uint32_t offset);

//------------------------------------------------
/* Delay Routine   */
//------------------------------------------------
void SLL_DelayMicroseconds(int microseconds);

#endif /* __SLL_AVALON_MBMC_REGS_H__ */
