/*
*  DISCLAIMER:
*
*      THIS SOFTWARE, SOURCE CODE AND ASSOCIATED MATERIALS INCLUDING BUT NOT LIMITED TO TUTORIALS,
*      GUIDES AND COMMENTARY PROVIDED WITH THIS EXERCISE ARE ONLY DESIGNED FOR REFERENCE PURPOSES
*      TO GIVE AN EXAMPLE TO LICENSEE FOR THEIR OWN NECESSARY DEVELOPMENT OF THEIR OWN SOFTWARE AND/OR
*      APPLICATION. IT IS NOT DESIGNED FOR ANY SPECIAL PURPOSE, SERIAL PRODUCTION OR USE IN MEDICAL,
*      MILITARY, AIR CRAFT, AVIATION, SPACE OF LIFE SUPPORT EQUIPMENT.
*
*      TO THE EXTENT PERMITTED BY LAW, THE EXERCISE SOFTWARE AND/OR SOURCE CODE AND/OR AND ASSOCIATED
*      MATERIALS IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND AND ONLY FOR REFERENCE PURPOSES.
*
*      SYNAPTIC LABORATORIES LTD. MAKES NO WARRANTIES, EITHER EXPRESS OR IMPLIED, WITH RESPECT TO THE
*      LICENSED SOFTWARE AND/OR SOURCE CODE AND/OR ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND
*      DOCUMENTATION PROVIDED HEREUNDER.�
*
*      SYNAPTIC LABORATORIES LTD. SPECIFICALLY DISCLAIMS THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
*      FITNESS FOR A PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT OF ANY INTELLECTUAL
*      PROPERTY RIGHT OF ANY THIRD PARTY WITH REGARD TO THE SOFTWARE, DOCUMENTATION (SCHEMATICS ETC.),
*      SOURCE CODE AND ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND DOCUMENTATION.
*
*      ANY USE, COMPILATION AND TESTING OF THE SOFTWARE AND/OR SOURCE CODE IS AT LICENSEE`S OWN RISK
*      AND LICENSEE IS OBLIGED TO CONDUCT EXTENSIVE TESTS TO AVOID ANY ERRORS AND FAILURE IN THE
*      COMPILED SOURCE CODE, DOCUMENTATION (SCHEMATICS ETC.) AND THE HEREFROM GENERATED SOFTWARE
*      OF LICENSEE.
*
*      EXCEPT FOR WILFULL INTENT SYNAPTIC LABORATORIES LTD. SHALL IN NO EVENT BE ENTITLED TO OR LIABLE
*      FOR ANY INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND OR NATURE, INCLUDING,
*      WITHOUT LIMITATION, BUSINESS INTERRUPTION COSTS, LOSS OF PROFIT OR REVENUE, LOSS OF DATA,
*      PROMOTIONAL OR MANUFACTURING EXPENSES, OVERHEAD, COSTS OR EXPENSES ASSOCIATED WITH WARRANTY
*      OR INTELLECTUAL PROPERTY INFRINGEMENT CLAIMS, INJURY TO REPUTATION OR LOSS OF CUSTOMERS.
*
*/

#include "sll_drvr_hyperflash.h"

#define NOR_FLASH_PAGE_SIZE (256)


/******************************************************************************
* 
* sll_nor_flash_drvr_ReadCFI - Gets CFI data
*
*
* RETURNS: CFI data 
*
*/
uint16_t sll_nor_flash_drvr_ReadCfiWord(uint32_t base_csr, uint32_t offset) {
	sll_hyperflash_write_cmd(base_csr, (offset & SA_OFFSET_MASK) + LLD_CFI_UNLOCK_ADDR1, NOR_CFI_QUERY_CMD);
	uint16_t data  = sll_hyperflash_read_cmd(base_csr, offset);
	sll_hyperflash_write_cmd(base_csr, 0, NOR_RESET_CMD); 
	return(data);
}



/******************************************************************************
* 
* sll_nor_flash_drvr_GetDeviceId - Get Flash device ID information
*
* RETURNS:  Flash ID information
*
*/
unsigned int sll_nor_flash_drvr_GetDeviceId(uint32_t base_csr, uint32_t offset) {
	unsigned int id;
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_UNLOCK_DATA1);
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR2, NOR_UNLOCK_DATA2);
	sll_hyperflash_write_cmd(base_csr, (offset & SA_OFFSET_MASK) + LLD_UNLOCK_ADDR1, NOR_AUTOSELECT_CMD);
	id  = (unsigned int)(sll_hyperflash_read_cmd(base_csr, (offset & SA_OFFSET_MASK) + 0x0001) & 0x000000FF) << 16;
	id |= (unsigned int)(sll_hyperflash_read_cmd(base_csr, (offset & SA_OFFSET_MASK) + 0x000E) & 0x000000FF) <<  8;
	id |= (unsigned int)(sll_hyperflash_read_cmd(base_csr, (offset & SA_OFFSET_MASK) + 0x000F) & 0x000000FF)      ;
	/* Write Software RESET command */
	sll_hyperflash_write_cmd(base_csr, 0, NOR_RESET_CMD);
	return(id);
}


/******************************************************************************
* 
* sll_nor_flash_drvr_Poll - Polls flash status device 
*
*
* RETURNS: value of status register
*
*/

uint16_t sll_nor_flash_drvr_Poll(uint32_t base_csr, uint32_t offset) {
	unsigned long polling_counter = 0xFFFFFFF;
	volatile uint16_t status_reg;
	do {
		polling_counter--;
		sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_STATUS_REG_READ_CMD);   /* Issue status register read command */
		status_reg = sll_hyperflash_read_cmd(base_csr, offset);       /* read the status register */
		if ((status_reg & DEV_RDY_MASK) == DEV_RDY_MASK) {  /* Are all devices done bit 7 is 1 */
			break;
		}
	} while (polling_counter);

	return status_reg;          /* retrun the status reg. */
}



/******************************************************************************
* 
* sll_nor_flash_drvr_WriteBufferProgramOp - Performs a Write Buffer Programming Operation.
*
* RETURNS: NOR_DEV_STATUS
*/
DEVSTATUS sll_nor_flash_drvr_WriteBufferProgramOp(uint32_t base_csr, uint32_t offset, uint32_t word_count, uint16_t *data_buf) {
	uint32_t  last_loaded_addr;
	uint32_t  current_offset;
	uint32_t  end_offset;
	uint16_t  wcount;
	uint16_t  status_reg;

	/* Initialize variables */
	current_offset   = offset;
	end_offset       = offset + word_count - 1;
	last_loaded_addr = offset;

	/* Check if number of words to program is zero */
	if (!word_count) {
		return DEV_NOT_BUSY;
	}

	/* Issue Load Write Buffer Command Sequence */
	/* Issue unlock command sequence */
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_UNLOCK_DATA1);
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR2, NOR_UNLOCK_DATA2);

	/* Write Write Buffer Load Command */
	sll_hyperflash_write_cmd(base_csr, offset, NOR_WRITE_BUFFER_LOAD_CMD);


	/* Write # of locations to program */
	wcount = (uint16_t)word_count - 1;

	/* In the datasheets of some latest Cypress devices, such as GLP, GLS, etc, the 
	command sequence of "write to buffer" command states the address of word count is 
	"Sector Address". Notice that to make LLD backward compatibility, the actual word 
	count address implemented is "Sector Address + LLD_UNLOCK_ADDR2", since the lower 
	address bits (a0-a15) are "don't care" bits and will be ignored anyway.
	*/
	sll_hyperflash_write_cmd(base_csr, offset & SA_OFFSET_MASK , wcount);

	/* Load Data into Buffer */
	while (current_offset <= end_offset) {
		/* Store last loaded address & data value (for polling) */
		last_loaded_addr = current_offset;

		/* Write Data */
		sll_hyperflash_write_cmd(base_csr, current_offset, *data_buf++);
		current_offset++;
	}

	/* Issue Program Buffer to Flash command */
	sll_hyperflash_write_cmd(base_csr, offset, NOR_WRITE_BUFFER_PGM_CONFIRM_CMD);

	/* Get Status */
	status_reg = sll_nor_flash_drvr_Poll(base_csr, last_loaded_addr);

	/* Check Lock Flag  */
	if (status_reg & DEV_SEC_LOCK_MASK) {
		return DEV_SECTOR_LOCK;      /* sector locked */
	}

	/* Check Program Flag  */
	if ((status_reg & DEV_PROGRAM_MASK) == DEV_PROGRAM_MASK) {
		return DEV_PROGRAM_ERROR;    /* program error */
	}

	return DEV_NOT_BUSY;           /* program complete */
}



/******************************************************************************
* 
* sll_nor_flash_drvr_SectorEraseOp - Performs a Sector Erase Operation 
*                                   (based on device sector size)
*
* RETURNS: NOR_DEV_STATUS
*
* ERRNO: 
*/


DEVSTATUS sll_nor_flash_drvr_SectorEraseOp(uint32_t base_csr, uint32_t offset) {
	uint16_t  status_reg;

	/* Issue erase command */
	/* Issue unlock sequence command */
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_UNLOCK_DATA1);
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR2, NOR_UNLOCK_DATA2);

	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_ERASE_SETUP_CMD);

	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR1, NOR_UNLOCK_DATA1);
	sll_hyperflash_write_cmd(base_csr, LLD_UNLOCK_ADDR2, NOR_UNLOCK_DATA2);

	/* Write Sector Erase Command to Offset */
	sll_hyperflash_write_cmd(base_csr, offset, NOR_SECTOR_ERASE_CMD);

	status_reg = sll_nor_flash_drvr_Poll(base_csr, offset );

	if (status_reg & DEV_SEC_LOCK_MASK) {
		return DEV_SECTOR_LOCK;    /* sector locked */
	}

	if ((status_reg & DEV_ERASE_MASK) == DEV_ERASE_MASK) {
		return DEV_ERASE_ERROR;    /* erase error */
	}
		
	return DEV_NOT_BUSY;         /* erease complete */
}


/******************************************************************************
* 
* sll_nor_flash_drvr_memcpy :  Performs program operation on a chunk of data
*                             Divides incoming data into a number of chunks
*                             Issues a write Buffer Program for each data chunk
*
* RETURNS: NOR_DEV_STATUS
*
*/

DEVSTATUS sll_nor_flash_drvr_memcpy(uint32_t base_csr, uint32_t offset, uint32_t word_cnt, uint16_t *data_buf) {
	uint32_t mask = NOR_FLASH_PAGE_SIZE - 1;
	uint32_t initial_count = word_cnt;
	DEVSTATUS status = DEV_NOT_BUSY;

	if (offset & mask) {
		/* check whether count is greater than buffer size */
		if (word_cnt < (NOR_FLASH_PAGE_SIZE - (offset & mask))) {
			initial_count = word_cnt;
		} else {
			initial_count = NOR_FLASH_PAGE_SIZE - (offset & mask);
		}

		/* program the first few to get write buffer aligned */
		status = sll_nor_flash_drvr_WriteBufferProgramOp(base_csr, offset, initial_count, data_buf);
		if (status != DEV_NOT_BUSY) {
			return status;
		}

		offset   += initial_count; /* adjust pointers and counter */
		word_cnt -= initial_count;
		data_buf += initial_count;
		if (word_cnt == 0) {
			return status;
		}
	}

	/* program next chunks */
	while (word_cnt >= NOR_FLASH_PAGE_SIZE) { /* while big chunks to do */
		status = sll_nor_flash_drvr_WriteBufferProgramOp(base_csr, offset, NOR_FLASH_PAGE_SIZE, data_buf);
		if (status != DEV_NOT_BUSY) {
			return status;
		}
		offset   += NOR_FLASH_PAGE_SIZE; /* adjust pointers and counter */
		word_cnt -= NOR_FLASH_PAGE_SIZE;
		data_buf += NOR_FLASH_PAGE_SIZE;
	}
	
	if (word_cnt == 0) {
		return status;
	}

	/* Write the remaining data if any*/
	status = sll_nor_flash_drvr_WriteBufferProgramOp(base_csr, offset, word_cnt, data_buf);
	return status;
}
